#include "calc.h"

#include <stdio.h>
#include <stdarg.h>
#include <time.h>
#include <sys/time.h>


static void fsm_debug( char* format, ... )
{
    struct timeval tval;
    struct tm t;
    gettimeofday(&tval,NULL);
    localtime_r(&tval.tv_sec, &t);

    printf("%02d:%02d:%02d.%lu ",
           t.tm_hour, t.tm_min, t.tm_sec, (unsigned long)tval.tv_usec);

    va_list ap;
    va_start( ap, format );
    vprintf( format, ap );
    va_end( ap );
    fflush( stdout );
}


void CalculatorFSM::__processEvent( Event e )
{
   States yOld = __Y;
   bool pass = false;
   fsm_debug( "CalculatorFSM:%#x:event:%s:begin\n", (int)this, eventName(e) );
   switch( __Y ) { //transitions 
   case Idle:
      if( e == Number ) {
         fsm_debug( "CalculatorFSM:%#x:state:Idle:leave:begin\n", (int)this );
         //outcome actions
         fsm_debug( "CalculatorFSM:%#x:state:Idle:leave:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:0:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:addNumber:begin\n", (int)this );
         addNumber();
         fsm_debug( "CalculatorFSM:%#x:output:addNumber:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:0:end\n", (int)this );
         __Y = EnteringFirstOperand;
         pass = true;
      }
      break;
   case EnteringFirstOperand:
      if( e == Number ) {
         fsm_debug( "CalculatorFSM:%#x:transition:0:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:addNumber:begin\n", (int)this );
         addNumber();
         fsm_debug( "CalculatorFSM:%#x:output:addNumber:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:0:end\n", (int)this );
         pass = false;
      }
      else if( e == BinaryOp ) {
         fsm_debug( "CalculatorFSM:%#x:state:EnteringFirstOperand:leave:begin\n", (int)this );
         //outcome actions
         fsm_debug( "CalculatorFSM:%#x:state:EnteringFirstOperand:leave:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:1:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:saveNumber:begin\n", (int)this );
         saveNumber();
         fsm_debug( "CalculatorFSM:%#x:output:saveNumber:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:saveOp:begin\n", (int)this );
         saveOp();
         fsm_debug( "CalculatorFSM:%#x:output:saveOp:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:1:end\n", (int)this );
         __Y = EnteringSecondOperand;
         pass = true;
      }
      else if( e == Clear ) {
         fsm_debug( "CalculatorFSM:%#x:state:EnteringFirstOperand:leave:begin\n", (int)this );
         //outcome actions
         fsm_debug( "CalculatorFSM:%#x:state:EnteringFirstOperand:leave:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:2:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:clear:begin\n", (int)this );
         clear();
         fsm_debug( "CalculatorFSM:%#x:output:clear:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:2:end\n", (int)this );
         __Y = Idle;
         pass = true;
      }
      break;
   case Result:
      if( e == Number ) {
         fsm_debug( "CalculatorFSM:%#x:state:Result:leave:begin\n", (int)this );
         //outcome actions
         fsm_debug( "CalculatorFSM:%#x:state:Result:leave:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:0:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:clear:begin\n", (int)this );
         clear();
         fsm_debug( "CalculatorFSM:%#x:output:clear:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:addNumber:begin\n", (int)this );
         addNumber();
         fsm_debug( "CalculatorFSM:%#x:output:addNumber:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:0:end\n", (int)this );
         __Y = EnteringFirstOperand;
         pass = true;
      }
      else if( e == BinaryOp ) {
         fsm_debug( "CalculatorFSM:%#x:state:Result:leave:begin\n", (int)this );
         //outcome actions
         fsm_debug( "CalculatorFSM:%#x:state:Result:leave:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:1:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:saveNumber:begin\n", (int)this );
         saveNumber();
         fsm_debug( "CalculatorFSM:%#x:output:saveNumber:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:saveOp:begin\n", (int)this );
         saveOp();
         fsm_debug( "CalculatorFSM:%#x:output:saveOp:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:1:end\n", (int)this );
         __Y = EnteringSecondOperand;
         pass = true;
      }
      else if( e == Clear ) {
         fsm_debug( "CalculatorFSM:%#x:state:Result:leave:begin\n", (int)this );
         //outcome actions
         fsm_debug( "CalculatorFSM:%#x:state:Result:leave:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:2:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:clear:begin\n", (int)this );
         clear();
         fsm_debug( "CalculatorFSM:%#x:output:clear:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:2:end\n", (int)this );
         __Y = Idle;
         pass = true;
      }
      break;
   case EnteringSecondOperand:
      if( e == Number ) {
         fsm_debug( "CalculatorFSM:%#x:transition:0:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:addNumber:begin\n", (int)this );
         addNumber();
         fsm_debug( "CalculatorFSM:%#x:output:addNumber:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:0:end\n", (int)this );
         pass = false;
      }
      else if( e == ResultRequested ) {
         fsm_debug( "CalculatorFSM:%#x:state:EnteringSecondOperand:leave:begin\n", (int)this );
         //outcome actions
         fsm_debug( "CalculatorFSM:%#x:state:EnteringSecondOperand:leave:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:1:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:1:end\n", (int)this );
         __Y = Result;
         pass = true;
      }
      else if( e == Clear ) {
         fsm_debug( "CalculatorFSM:%#x:state:EnteringSecondOperand:leave:begin\n", (int)this );
         //outcome actions
         fsm_debug( "CalculatorFSM:%#x:state:EnteringSecondOperand:leave:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:2:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:clear:begin\n", (int)this );
         clear();
         fsm_debug( "CalculatorFSM:%#x:output:clear:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:2:end\n", (int)this );
         __Y = Idle;
         pass = true;
      }
      else if( e == BinaryOp ) {
         fsm_debug( "CalculatorFSM:%#x:state:EnteringSecondOperand:leave:begin\n", (int)this );
         //outcome actions
         fsm_debug( "CalculatorFSM:%#x:state:EnteringSecondOperand:leave:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:transition:3:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:self event:BinaryOp\n", (int)this );
         this->A( BinaryOp );
         fsm_debug( "CalculatorFSM:%#x:transition:3:end\n", (int)this );
         __Y = Result;
         pass = true;
      }
      break;
   }

   if( yOld == __Y && !pass ) {
      fsm_debug( "CalculatorFSM:%#x:event:%s:end\n", (int)this, eventName(e) );
      return;
   }

   switch( __Y ) { // income actions
   case Idle:
         fsm_debug( "CalculatorFSM:%#x:state:Idle:enter:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:state:Idle:enter:end\n", (int)this );
      break;
   case EnteringFirstOperand:
         fsm_debug( "CalculatorFSM:%#x:state:EnteringFirstOperand:enter:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:state:EnteringFirstOperand:enter:end\n", (int)this );
      break;
   case Result:
         fsm_debug( "CalculatorFSM:%#x:state:Result:enter:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:output:showResult:begin\n", (int)this );
         showResult();
         fsm_debug( "CalculatorFSM:%#x:output:showResult:end\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:state:Result:enter:end\n", (int)this );
      break;
   case EnteringSecondOperand:
         fsm_debug( "CalculatorFSM:%#x:state:EnteringSecondOperand:enter:begin\n", (int)this );
         fsm_debug( "CalculatorFSM:%#x:state:EnteringSecondOperand:enter:end\n", (int)this );
      break;
   }
      fsm_debug( "CalculatorFSM:%#x:event:%s:end\n", (int)this, eventName(e) );
}

void CalculatorFSM::A( Event e )
{
   bool __empty = __events.empty();
   __events.push( e );
   if( __empty ) {
      while( !__events.empty() ) {
         __processEvent( __events.front() );
         __events.pop();
      }
   }
}

bool CalculatorFSM::debug_iNtErNal_input( const char*, const char* name, bool i)
{
   fsm_debug( "CalculatorFSM:%#x:input:%s:%s\n", (int)this, name, i ? "true" : "false" );
   return i;
}   
const char* CalculatorFSM::eventName( Event e )
{
   switch(e) {
   case ResultRequested:
      return "ResultRequested";
   case Number:
      return "Number";
   case BinaryOp:
      return "BinaryOp";
   case Clear:
      return "Clear";
   default:
      return "Unknown event";
   }
}

const char* CalculatorFSM::stateName( States s )
{
   switch(s) {
   case Idle:
      return "Idle";
   case EnteringFirstOperand:
      return "EnteringFirstOperand";
   case Result:
      return "Result";
   case EnteringSecondOperand:
      return "EnteringSecondOperand";
   default:
      return "Unknown state";
   }
}
void CalculatorFSM::debug_show_fsm_xml()
{
   printf( "\nBegin FSM XML:CalculatorFSM\n%s\n\nEnd FSM XML\n\n", 
"<!DOCTYPE FSM> <FSM> <statemachine> <name>CalculatorFSM</name> <comment>Quite useless calculator</comment> <initialstate>Idle</initialstate> <event> <name>ResultRequested</name> <comment>A '=' have been entered</comment> </event> <event> <name>Number</name> <comment>A digit have been entered</comment> </event> <event> <name>BinaryOp</name> <comment>Pressed '+', '-', ...</comment> </event> <event> <name>Clear</name> <comment>User have pressed 'C' button. Clear all data</comment> </event> <output> <name>saveNumber</name> <comment>A number have been entered, save it and proceed to the next one</comment> </output> <output> <name>addNumber</name> <comment>Add a number to the current number, for example, if there is '12' on a screen and '3' button pressed, there should be '123' on a screen</comment> </output> <output> <name>clear</name> <comment>Just clear :)</comment> </output> <output> <name>saveOp</name> <comment>Save operation to our stack. This output follows press '+', '-', ...</comment> </output> <output> <name>showResult</name> <comment>Calculate and display result of operation</comment> </output> <state> <name>Idle</name> <comment>A result is displayed</comment> <incomeactions/> <outcomeactions/> <transitions> <transition type=\"simple\" > <name>start</name> <comment>User started to enter a number</comment> <condition>Number</condition> <destination>EnteringFirstOperand</destination> <transite_destination/> <transitionactions> <action type=\"output\" >addNumber</action> </transitionactions> </transition> </transitions> </state> <state> <name>EnteringFirstOperand</name> <comment>A first calculating operand is being entered</comment> <incomeactions/> <outcomeactions/> <transitions> <transition type=\"simple\" > <name>add</name> <comment/> <condition>Number</condition> <destination/> <transite_destination/> <transitionactions> <action type=\"output\" >addNumber</action> </transitionactions> </transition> <transition type=\"simple\" > <name>op</name> <comment/> <condition>BinaryOp</condition> <destination>EnteringSecondOperand</destination> <transite_destination/> <transitionactions> <action type=\"output\" >saveNumber</action> <action type=\"output\" >saveOp</action> </transitionactions> </transition> <transition type=\"simple\" > <name>clear</name> <comment>start from scratch</comment> <condition>Clear</condition> <destination>Idle</destination> <transite_destination/> <transitionactions> <action type=\"output\" >clear</action> </transitionactions> </transition> </transitions> </state> <state> <name>Result</name> <comment>A result is displayed, but there is a result from previous operation</comment> <incomeactions> <action type=\"output\" >showResult</action> </incomeactions> <outcomeactions/> <transitions> <transition type=\"simple\" > <name>from scratch</name> <comment>If user presses a number button when there's a result on a screen, we should clear it and start from scratch</comment> <condition>Number</condition> <destination>EnteringFirstOperand</destination> <transite_destination/> <transitionactions> <action type=\"output\" >clear</action> <action type=\"output\" >addNumber</action> </transitionactions> </transition> <transition type=\"simple\" > <name>next chain</name> <comment>If user presses an action button (+, -, *, /), we should remember it and ask for the second operand</comment> <condition>BinaryOp</condition> <destination>EnteringSecondOperand</destination> <transite_destination/> <transitionactions> <action type=\"output\" >saveNumber</action> <action type=\"output\" >saveOp</action> </transitionactions> </transition> <transition type=\"simple\" > <name>clear</name> <comment/> <condition>Clear</condition> <destination>Idle</destination> <transite_destination/> <transitionactions> <action type=\"output\" >clear</action> </transitionactions> </transition> </transitions> </state> <state> <name>EnteringSecondOperand</name> <comment>User is pressing digits to make second operand</comment> <incomeactions/> <outcomeactions/> <transitions> <transition type=\"simple\" > <name>add</name> <comment/> <condition>Number</condition> <destination/> <transite_destination/> <transitionactions> <action type=\"output\" >addNumber</action> </transitionactions> </transition> <transition type=\"simple\" > <name>result</name> <comment>User pressed '=', we should calculate and show result</comment> <condition>ResultRequested</condition> <destination>Result</destination> <transite_destination/> <transitionactions/> </transition> <transition type=\"simple\" > <name>clear</name> <comment/> <condition>Clear</condition> <destination>Idle</destination> <transite_destination/> <transitionactions> <action type=\"output\" >clear</action> </transitionactions> </transition> <transition type=\"simple\" > <name>next op</name> <comment>Pressing operation button is equal to pressing '=' and then this button</comment> <condition>BinaryOp</condition> <destination>Result</destination> <transite_destination/> <transitionactions> <action type=\"self_event\" >BinaryOp</action> </transitionactions> </transition> </transitions> </state> </statemachine> <graphics> <class classname=\"GStateSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GStateSaver\" > <qstring name=\"stateName\" >Idle</qstring> <QPoint name=\"position\" > <int>5042</int> <int>5045</int> </QPoint> </data> </class> <class classname=\"GStateSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GStateSaver\" > <qstring name=\"stateName\" >EnteringSecondOperand</qstring> <QPoint name=\"position\" > <int>5289</int> <int>5171</int> </QPoint> </data> </class> <class classname=\"GStateSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GStateSaver\" > <qstring name=\"stateName\" >Result</qstring> <QPoint name=\"position\" > <int>5259</int> <int>5034</int> </QPoint> </data> </class> <class classname=\"GStateSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GStateSaver\" > <qstring name=\"stateName\" >EnteringFirstOperand</qstring> <QPoint name=\"position\" > <int>5046</int> <int>5174</int> </QPoint> </data> </class> <class classname=\"GTransitionSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GTransitionSaver\" > <qstring name=\"stateName\" >Idle</qstring> <int name=\"transition\" >0</int> <qvaluelist name=\"points\" /> </data> </class> <class classname=\"GTransitionSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GTransitionSaver\" > <qstring name=\"stateName\" >EnteringFirstOperand</qstring> <int name=\"transition\" >0</int> <qvaluelist name=\"points\" > <QPoint> <int>5051</int> <int>5237</int> </QPoint> <QPoint> <int>5089</int> <int>5255</int> </QPoint> </qvaluelist> </data> </class> <class classname=\"GTransitionSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GTransitionSaver\" > <qstring name=\"stateName\" >EnteringFirstOperand</qstring> <int name=\"transition\" >1</int> <qvaluelist name=\"points\" > <QPoint> <int>5206</int> <int>5225</int> </QPoint> <QPoint> <int>5276</int> <int>5225</int> </QPoint> </qvaluelist> </data> </class> <class classname=\"GTransitionSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GTransitionSaver\" > <qstring name=\"stateName\" >EnteringSecondOperand</qstring> <int name=\"transition\" >0</int> <qvaluelist name=\"points\" > <QPoint> <int>5451</int> <int>5214</int> </QPoint> <QPoint> <int>5425</int> <int>5247</int> </QPoint> </qvaluelist> </data> </class> <class classname=\"GTransitionSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GTransitionSaver\" > <qstring name=\"stateName\" >EnteringSecondOperand</qstring> <int name=\"transition\" >1</int> <qvaluelist name=\"points\" > <QPoint> <int>5355</int> <int>5108</int> </QPoint> </qvaluelist> </data> </class> <class classname=\"GTransitionSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GTransitionSaver\" > <qstring name=\"stateName\" >Result</qstring> <int name=\"transition\" >0</int> <qvaluelist name=\"points\" > <QPoint> <int>5171</int> <int>5090</int> </QPoint> </qvaluelist> </data> </class> <class classname=\"GTransitionSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GTransitionSaver\" > <qstring name=\"stateName\" >Result</qstring> <int name=\"transition\" >1</int> <qvaluelist name=\"points\" > <QPoint> <int>5273</int> <int>5112</int> </QPoint> <QPoint> <int>5278</int> <int>5138</int> </QPoint> </qvaluelist> </data> </class> <class classname=\"GTransitionSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GTransitionSaver\" > <qstring name=\"stateName\" >EnteringFirstOperand</qstring> <int name=\"transition\" >2</int> <qvaluelist name=\"points\" > <QPoint> <int>5035</int> <int>5131</int> </QPoint> </qvaluelist> </data> </class> <class classname=\"GTransitionSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GTransitionSaver\" > <qstring name=\"stateName\" >EnteringSecondOperand</qstring> <int name=\"transition\" >2</int> <qvaluelist name=\"points\" > <QPoint> <int>5316</int> <int>5263</int> </QPoint> <QPoint> <int>5105</int> <int>5305</int> </QPoint> <QPoint> <int>4996</int> <int>5258</int> </QPoint> </qvaluelist> </data> </class> <class classname=\"GTransitionSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GTransitionSaver\" > <qstring name=\"stateName\" >Result</qstring> <int name=\"transition\" >2</int> <qvaluelist name=\"points\" > <QPoint> <int>5155</int> <int>5032</int> </QPoint> </qvaluelist> </data> </class> <class classname=\"GTransitionSaver\" > <meta classname=\"GSaver\" > <data classname=\"GSaver\" /> </meta> <data classname=\"GTransitionSaver\" > <qstring name=\"stateName\" >EnteringSecondOperand</qstring> <int name=\"transition\" >3</int> <qvaluelist name=\"points\" > <QPoint> <int>5426</int> <int>5128</int> </QPoint> <QPoint> <int>5367</int> <int>5061</int> </QPoint> </qvaluelist> </data> </class> </graphics> </FSM>"   );
   fflush(stdout);
}
