#ifndef CalculatorFSM283545993
#define CalculatorFSM283545993
#include <queue> // for event queue


/**
Quite useless calculator
*/
class CalculatorFSM {
public:
/** Events*/
   enum Event { ResultRequested, /*!< A '=' have been entered*/
                Number, /*!< A digit have been entered*/
                BinaryOp, /*!< Pressed '+', '-', ...*/
                Clear/*!< User have pressed 'C' button. Clear all data*/
   };
protected:


/**
A number have been entered, save it
and proceed to the next one
*/
   virtual void saveNumber() = 0; 
/**
Add a number to the current number,
for example, if there is '12' on a screen
and '3' button pressed, there should
be '123' on a screen
*/
   virtual void addNumber() = 0; 
/**
Just clear :)
*/
   virtual void clear() = 0; 
/**
Save operation to our stack.
This output follows press '+', '-', ...
*/
   virtual void saveOp() = 0; 
/**
Calculate and display result of operation
*/
   virtual void showResult() = 0; 

public:
  /** States */
   enum States { Idle, /*!< A result is displayed*/
               EnteringFirstOperand, /*!< A first calculating operand is being entered*/
               Result, /*!< A result is displayed, but there is a result from previous operation*/
               EnteringSecondOperand/*!< User is pressing digits to make second operand*/
   }; // states

private:
   States __Y;
   std::queue<Event> __events; 
   void __processEvent( Event e );
   bool debug_iNtErNal_input(const char* type, const char* name, bool i);
   void debug_show_fsm_xml();
public:
   /// Constructor
   CalculatorFSM() { __Y = Idle;
   debug_show_fsm_xml();
   }
   /// Destructor
   virtual ~CalculatorFSM() {}
   /** Get current FSM state
       @returns current FSM state
    */
   States currentState() { return __Y; }


   /** Get symbolic name of an event

      @param e - numeric event
      @returns symbolic event name
    */
   static const char* eventName( Event e );


   /** Get symbolic name of a state

      @param e - numeric state
      @returns symbolic state name
    */
   
   static const char* stateName( States s );


   /** Send event to FSM

       Use this function to send event to CalculatorFSM
       After you call it given event will be handled,
       and, if some of transition conditions match, 
       appropriate transition will be triggered,
       and currentState() will be changed. 

       If this function is called during existing event
       handling process, given event will be added to 
       pending event queue, and will be handled after current
       transition. See examples for details.
   */
   void A( Event e );
};

#endif
